<?php
/**
* Class to provide IPv4 calculations
*
* PHP versions 4 and 5
*
* LICENSE: This source file is subject to version 3.01 of the PHP license
* that is available through the world-wide-web at the following URI:
* http://www.php.net/license/3_01.txt.  If you did not receive a copy of
* the PHP License and are unable to obtain it through the web, please
* send a note to license@php.net so we can mail you a copy immediately.
*
* @category   Net
* @package    Net_IPv4
* @author     Eric Kilfoil <edk@ypass.net>
* @author     Marco Kaiser <bate@php.net>
* @author     Florian Anderiasch <fa@php.net>
* @copyright  1997-2005 The PHP Group
* @license    http://www.php.net/license/3_01.txt  PHP License 3.01
* @version    CVS: $Id: IPv4.php,v 1.11 2005/11/29 12:56:35 fa Exp $
* @link       http://pear.php.net/package/Net_IPv4
* 
* @barely modified by SharkyDog
*/

class Net_IPv4 {
    var $error = "";
    var $netmaskmap = array(
            0 => "0.0.0.0",
            1 => "128.0.0.0",
            2 => "192.0.0.0",
            3 => "224.0.0.0",
            4 => "240.0.0.0",
            5 => "248.0.0.0",
            6 => "252.0.0.0",
            7 => "254.0.0.0",
            8 => "255.0.0.0",
            9 => "255.128.0.0",
            10 => "255.192.0.0",
            11 => "255.224.0.0",
            12 => "255.240.0.0",
            13 => "255.248.0.0",
            14 => "255.252.0.0",
            15 => "255.254.0.0",
            16 => "255.255.0.0",
            17 => "255.255.128.0",
            18 => "255.255.192.0",
            19 => "255.255.224.0",
            20 => "255.255.240.0",
            21 => "255.255.248.0",
            22 => "255.255.252.0",
            23 => "255.255.254.0",
            24 => "255.255.255.0",
            25 => "255.255.255.128",
            26 => "255.255.255.192",
            27 => "255.255.255.224",
            28 => "255.255.255.240",
            29 => "255.255.255.248",
            30 => "255.255.255.252",
            31 => "255.255.255.254",
            32 => "255.255.255.255"
    );    

    function validateIP($ip) {
        if($ip == long2ip(ip2long($ip))) return true;
        else return false;
    }

    function validateNetmask($netmask) {
        if(!in_array($netmask, $this->netmaskmap)) return false;
        return true;
    }

    function parseAddress($address) {
        $return = array();
        $this->error = "";
        
        if(strchr($address, "/")) {
            $parts = explode("/", $address);

            if(!$this->validateIP($parts[0])) {
                $this->error = "invalid IP address";
                return false;
            }
            $return["ip"] = $parts[0];

            // Check the style of netmask that was entered 
            if(eregi("^([0-9a-f]{2})([0-9a-f]{2})([0-9a-f]{2})([0-9a-f]{2})$", $parts[1], $regs)) {
                // hexadecimal string
                $return["netmask"] = hexdec($regs[1]).".".hexdec($regs[2]).".".hexdec($regs[3]).".".hexdec($regs[4]);
            }
            else if(strchr($parts[1], ".")) {
                // a standard dot quad netmask was entered.
                if(!$this->validateNetmask($parts[1])) {
                    $this->error = "invalid netmask value";
                    return false;
                }
                $return["netmask"] = $parts[1];
            }
            else if($parts[1] >= 0 && $parts[1] <= 32) {
                // a CIDR bitmask type was entered
                $return["bitmask"] = $parts[1];
            }
            else {
                // Some unknown format of netmask was entered
                $this->error = "invalid netmask value";
                return false;
            }
            
            $return = $this->calculate($return);
            return $return;
        }
        else if($this->validateIP($address)) {
            $return["ip"] = $address;
            return $return;
        }
        else {
            $this->error = "invalid IP address";
            return false;
        }
    }
    
    function calculate($data) {
        $validNM = $this->netmaskmap;
        $this->error = "";

        // Find out if we were given an ip address in dot quad notation or
        // a network long ip address.  Whichever was given, populate the
        // other field
        if(strlen($data["ip"])) {
            if(!$this->validateIP($data["ip"])) {
                $this->error = "invalid IP address";
                return false;
            }
            $data["long"] = $this->ip2double($data["ip"]);
        }
        else if(is_numeric($data["long"])) {
            $data["ip"] = long2ip($data["long"]);
        }
        else {
           $this->error = "ip address not specified";
           return false;
        }

        // Check to see if we were supplied with a bitmask or a netmask.
        // Populate the other field as needed.
        if(strlen($data["bitmask"])) {
            $data["netmask"] = $validNM[$data["bitmask"]];
        }
        else if(strlen($data["netmask"])) {
            $validNM_rev = array_flip($validNM);
            $data["bitmask"] = $validNM_rev[$data["netmask"]];
        }
        else {
            $this->error = "netmask or bitmask are required for calculation";
            return false;
        }
        $data["network"] = long2ip(ip2long($data["ip"]) & ip2long($data["netmask"]));
        $data["broadcast"] = long2ip( ip2long($data["ip"]) | (ip2long($data["netmask"]) ^ ip2long("255.255.255.255")) );
        return $data;
    }
    
	  function getNetmask($length) {
	      $return = $this->parseAddress("0.0.0.0/".$length);
		    if(!$return) return false;
		    else return $return["netmask"];
  	}

  	function getNetLength($netmask) {
  	    $return = $this->parseAddress("0.0.0.0/".$netmask);
		    if(!$return) return false;
		    else return $return["bitmask"];
  	}
    
	  function getSubnet($ip, $netmask) {
	      $return = $this->parseAddress($ip."/".$netmask);
		    if(!$return) return false;
		    else return $return["network"];
  	}
    
	  function inSameSubnet($ip1, $ip2) {
		    $retip1 = $this->parseAddress($ip1);
			  if(!$retip1) return false;
		    $retip2 = $this->parseAddress($ip2);
		    if(!$retip2) return false;

		    if($retip1["network"]==$retip2["network"] && $retip1["bitmask"]==$retip1["bitmask"]) return true;
		    else return false;
    }
    
    function atoh($addr) {
        if(!$this->validateIP($addr)) return false;
        $ap = explode(".", $addr);
        return sprintf("%02x%02x%02x%02x", $ap[0], $ap[1], $ap[2], $ap[3]);
    }

    function htoa($addr) {
        if(eregi("^([0-9a-f]{2})([0-9a-f]{2})([0-9a-f]{2})([0-9a-f]{2})$", $addr, $regs)) {
            return hexdec($regs[1]).".".hexdec($regs[2]).".".hexdec($regs[3]).".".hexdec($regs[4]);
        }
        else return false;
    }

    function ip2double($ip) {
        return (double)(sprintf("%u", ip2long($ip)));
    }

    function ipInNetwork($ip, $network)
    {
        $return = $this->parseAddress($network);
        if(!$return) return false;
        
        $net = $this->ip2double($return["network"]);
        $bcast = $this->ip2double($return["broadcast"]);
        $ipl = $this->ip2double($ip);

        if($ipl >= $net && $ipl <= $bcast) return true;
        else return false;
    }
}
?>
