<?php
/*
  e107 website system
  http://e107.org

  NET/IP Filter

  $Revision: 494 $
  $Date: 2007-09-08 02:48:34 +0300 (събота, 08 Септември 2007) $
  $Author: sharkydog $
  Website: http://www.sharkydog.info
  E-mail: sharkydog@e107bg.org
*/

if (!defined('e107_INIT')) { exit; }

class NetIP_Filter {
  var $ver;
  var $active;
  var $denymess;
  var $def_deny_mess;
  var $core_events;
  var $plug_events;
  var $netcalc;
  
  function NetIP_Filter() {
    global $pref;
    
    $this->ver = $pref["nipf_version"];
    $this->active = $pref["nipf_active"];
    
    $this->def_deny_mess = str_replace(array("&#092;","&quot;","&#039;"),array("\\","'","\""),$pref["nipf_def_deny_mess"]);
    
    require(e_PLUGIN."netip_filter/accept_events.php");
    $this->core_events = is_array($nipf_core_evts) ? $nipf_core_evts : array();
    $this->plug_events = is_array($nipf_plug_evts) ? $nipf_plug_evts : array();
        
    if(!class_exists("Net_IPv4")) require_once(e_PLUGIN."netip_filter/net_ipv4.php");
    $this->netcalc = new Net_IPv4;
  }


  // Check Network and IP filter for denial
  //
  // $ip     - The IP address to check
  // $event  - Desired user defined event, string
  //
  // @return true on denial or false
  function check($ip,$event) {
    global $pref, $e_event, $tp, $sql;
    
    $netdeny = false;
    $netret = false;
    $ipret = false;
    $event_pref = false;
    $this->denymess = "";
    if(!$this->active || !$ip || !$event) return false;
    
    if($ip != long2ip(ip2long($ip))) {
      $this->denymess = "Invalid IP address!";
      if($event!="init") {
        $e_data = array(
          "ip_addr"=>$ip,
          "nipf_event"=>$event,
          "blocked_by"=>"invalid_ip",
          "blocked_by_data"=>false
        );
        $e_event->trigger("nipfblock", $e_data);
      }
      return true;
    }
    
    if($event=="none" || $event=="all") return false;
    if((!strpos($event,"eevt_") && strpos($event,"eevt_")!==false) || $event=="init") {
      if($event!="init") {
        $core_event = substr($event,5);
        if(in_array($core_event,$this->core_events)) {
          $event_pref = $pref["nipf_pre_nftype"][$event] ? $pref["nipf_pre_nftype"][$event] : $pref["nipf_nf_type"];
        }
        else return false;
      }
      else $event_pref = $pref["nipf_pre_nftype"][$event] ? $pref["nipf_pre_nftype"][$event] : $pref["nipf_nf_type"];
    } else {
      $r = $sql->db_Select("nipf_events","evt_nftype","evt_event='".$tp->toDB($event)."'");
      if($r) {
        $data = $sql->db_Fetch();
        $event_pref = $data["evt_nftype"];
      }
      else return false;
    }
    
    // Network filter
    if($event_pref=="all") $netdeny = false;
    else if($event_pref=="none") {
      $this->denymess = $this->def_deny_mess;
      $netdeny = true;
    }
    else {
      $retnf = $this->ret_nf($ip,"y",$event,1);
      if(is_array($retnf)) {
        if($event_pref=="all-list") {
          $this->denymess = $retnf[0]["denymess"] ? $retnf[0]["denymess"] : $this->def_deny_mess;
          $netdeny = true;
        }
        else $netdeny = false;
      }
      else {
        if($event_pref=="all-list") $netdeny = false;
        else {
          $this->denymess = $this->def_deny_mess;
          $netdeny = true;
        }
      }
    }
    
    // IP filter
    $retipf = $this->ret_ipf($ip,($netdeny ? "allow" : "deny"),$event,1);
    
    if(is_array($retipf)) {
      if($netdeny) {
        $this->denymess = "";
        return false;
      } else {
        $this->denymess = $retipf[0]["denymess"] ? $retipf[0]["denymess"] : $this->def_deny_mess;
        if($event!="init") {
          $e_data = array(
            "ip_addr"=>$ip,
            "nipf_event"=>$event,
            "blocked_by"=>"ip_filter",
            "blocked_by_data"=>$retipf[0]
          );
          $e_event->trigger("nipfblock", $e_data);
        }
        return true;
      }
    } else {
      if($netdeny && $event!="init") {
        $e_data = array(
          "ip_addr"=>$ip,
          "nipf_event"=>$event,
          "blocked_by"=>"net_filter",
          "blocked_by_data"=>is_array($retnf) ? $retnf[0] : array("id"=>false,"event_pref"=>$event_pref,"denymess"=>$this->denymess)
        );
        $e_event->trigger("nipfblock", $e_data);
      }
      return $netdeny;
    }
  }
  
  // Check if an IP is in the network filter and return found data
  //
  // $ip - The IP address
  // $active - Active [y,n]
  // $event - Desired user defined event, string
  // $limit - Limit returned networks, integer
  //
  // @return array or false
  function ret_nf($ip,$active=false,$event=false,$limit=false) {
    global $sql, $tp;
    
    if(!$this->active || $ip!=long2ip(ip2long($ip)) || ($active && !in_array($active,array("y","n")))) return false;
    if($event=="all" || $event=="none") return false;
    if($limit) $limit = intval($limit);

    $ipn = $this->netcalc->ip2double($ip);
    $event = $tp->toDB($event);
    $rn = $sql->db_Select("nipf_netfilter","*","nf_netlong<=".$ipn." AND ".$ipn."<=nf_bcastlong".($active ? " AND nf_active='".$active."'" : "").($event ? " AND (nf_event LIKE '%".$event.",%' OR nf_event LIKE '%,".$event."%' OR nf_event='".$event."' OR nf_event='all')" : "").($limit ? " LIMIT ".$limit : ""));
    if($rn) {
      $return = array();
      while($nf_data = $sql->db_Fetch()) {
        $return[] = array(
          "id"         =>  $nf_data["nf_id"],
          "bitmask"    =>  $nf_data["nf_bitmask"],
          "netmask"    =>  $nf_data["nf_netmask"],
          "network"    =>  $nf_data["nf_network"],
          "broadcast"  =>  $nf_data["nf_broadcast"],
          "netlong"    =>  $nf_data["nf_netlong"],
          "bcastlong"  =>  $nf_data["nf_bcastlong"],
          "netname"    =>  $nf_data["nf_netname"],
          "addinfo"    =>  $nf_data["nf_addinfo"],
          "active"     =>  $nf_data["nf_active"],
          "event"      =>  $nf_data["nf_event"],
          "denymess"   =>  $nf_data["nf_denymess"]
        );
      }
      return $return;
    }
    else return false;
  }
  
  // Check if an IP is in the IP filter and return found data
  //
  // $ip - The IP address
  // $action - none, allow or deny
  // $event - Desired user defined event, string
  // $limit - Limit returned IPs, integer
  //
  // @return array or false
  function ret_ipf($ip,$action=false,$event=false,$limit=false) {
    global $sql, $tp;
    
    if(!$this->active || $ip!=long2ip(ip2long($ip)) || ($action && !in_array($action,array("none","allow","deny")))) return false;
    if($event=="all" || $event=="none") return false;
    if($limit) $limit = intval($limit);

    $ipa = array();
    $ipi = explode(".",$ip);
    for($i=0;$i<4;$i++) {
      $prev = $i-1;
      if($prev<0) $prev = false;
      $ipa[$i] = ($prev!==false&&isset($ipa[$prev]) ? $ipa[$prev]."." : "").$ipi[$i];
    }
    $ipi = "'".implode("','",$ipa)."'";
    $event = $tp->toDB($event);
    $ri = $sql->db_Select("nipf_ipfilter","*","ipf_ip IN (".$ipi.")".($action ? " AND ipf_action='".$action."'" : "").($event ? " AND (ipf_event LIKE '%".$event.",%' OR ipf_event LIKE '%,".$event."%' OR ipf_event='".$event."' OR ipf_event='all')" : "").($limit ? " LIMIT ".$limit : ""));
    if($ri) {
      $return = array();
      while($ipf_data = $sql->db_Fetch()) {
        $return[] = array(
          "id"        =>  $ipf_data["ipf_id"],
          "ip"        =>  $ipf_data["ipf_ip"],
          "info"      =>  $ipf_data["ipf_info"],
          "action"    =>  $ipf_data["ipf_action"],
          "event"     =>  $ipf_data["ipf_event"],
          "denymess"  =>  $ipf_data["ipf_denymess"]
        );
      }
      return $return;
    }
    else return false;
  }

  // Check if a network is in the network filter and return found data
  //
  // $net - The network, format: Network (IP) address/CIDR bitmask or Network (IP) address/netmask - 192.168.0.0/24 or 192.168.0.0/255.255.255.0
  // $active - Active [y,n]
  // $event - Desired user defined event, string
  // $limit - Limit returned networks, integer
  //
  // @return array or false
  function in_nf($net,$active=false,$event=false,$limit=false) {
    global $sql, $tp;

    if(!$this->active || ($active && !in_array($active,array("y","n")))) return false;
    if($event=="all" || $event=="none") return false;
    if($limit) $limit = intval($limit);
    
    $retnet = $this->netcalc->parseAddress($net);
    if(!$netret || !$netret["bitmask"] || !$netret["network"]) return false;
    
    $event = $tp->toDB($event);
    $rn = $sql->db_Select("nipf_netfilter","*","nf_bitmask='".$netret["bitmask"]."' AND nf_network='".$netret["network"]."'".($active ? " AND nf_active='".$active."'" : "").($event ? " AND (nf_event LIKE '%".$event.",%' OR nf_event LIKE '%,".$event."%' OR nf_event='".$event."' OR nf_event='all')" : "").($limit ? " LIMIT ".$limit : ""));
    if($rn) {
      $return = array();
      while($nf_data = $sql->db_Fetch()) {
        $return[] = array(
          "id"         =>  $nf_data["nf_id"],
          "bitmask"    =>  $nf_data["nf_bitmask"],
          "netmask"    =>  $nf_data["nf_netmask"],
          "network"    =>  $nf_data["nf_network"],
          "broadcast"  =>  $nf_data["nf_broadcast"],
          "netlong"    =>  $nf_data["nf_netlong"],
          "bcastlong"  =>  $nf_data["nf_bcastlong"],
          "netname"    =>  $nf_data["nf_netname"],
          "addinfo"    =>  $nf_data["nf_addinfo"],
          "active"     =>  $nf_data["nf_active"],
          "event"      =>  $nf_data["nf_event"],
          "denymess"   =>  $nf_data["nf_denymess"]
        );
      }
      return $return;
    }
    else return false;
  }

  // Check if an IP is in the IP filter and return found data
  //
  // $ipchunk - The IP address or part of it, this function searches for exact match of this, format: 111.112.113.114 or 111.112.113 or 111.112 or 111
  // $action - none, allow or deny
  // $event - Desired user defined event, string
  // $limit - Limit returned IPs, integer
  //
  // @return array or false
  function in_ipf($ipchunk,$action=false,$event=false,$limit=false) {
    global $sql, $tp;
    
    if(!$this->active || ($action && !in_array($action,array("none","allow","deny")))) return false;
    if($event=="all" || $event=="none") return false;
    if($limit) $limit = intval($limit);
  
    $tmp = explode(".",$ipchunk);
    $val = "";
    if(count($tmp)>4) return false;
    else {
      foreach($tmp as $val) {
        if(!is_numeric($val) || $val<0 || $val>255) return false;
      }
    }
    
    $event = $tp->toDB($event);
    $ri = $sql->db_Select("nipf_ipfilter","*","ipf_ip='".$ipchunk."'".($action ? " AND ipf_action='".$action."'" : "").($event ? " AND (ipf_event LIKE '%".$event.",%' OR ipf_event LIKE '%,".$event."%' OR ipf_event='".$event."' OR nf_event='all')" : "").($limit ? " LIMIT ".$limit : ""));
    if($ri) {
      $return = array();
      while($ipf_data = $sql->db_Fetch()) {
        $return[] = array(
          "id"        =>  $ipf_data["ipf_id"],
          "ip"        =>  $ipf_data["ipf_ip"],
          "info"      =>  $ipf_data["ipf_info"],
          "action"    =>  $ipf_data["ipf_action"],
          "event"     =>  $ipf_data["ipf_event"],
          "denymess"  =>  $ipf_data["ipf_denymess"]
        );
      }
      return $return;
    }
    else return false;
  }
}
?>