<?php
/*
  e107 website system
  http://e107.org

  NET/IP Filter

  $Revision: 494 $
  $Date: 2007-09-08 02:48:34 +0300 (събота, 08 Септември 2007) $
  $Author: sharkydog $
  Website: http://www.sharkydog.info
  E-mail: sharkydog@e107bg.org
*/

if (!defined('e107_INIT')) { exit; }

class NIPF_Manage {
  var $error_code;
  var $error_mess;
  var $errors;
  var $netcalc;
  var $reset_event;
  var $core_events;
  var $plug_events;
  
  function NIPF_Manage() {
    global $pref, $nipf;
    
    $this->errors = array(
      0=>"",
      111=>"General: Command not recognized ({COMMAND})",
      121=>"MySQL: db_Insert() - {MERROR}",
      122=>"MySQL: db_Update() - {MERROR}",
      123=>"MySQL: db_Delete() - {MERROR}",
      124=>"MySQL: db_Select() - {MERROR}",
      211=>"NetAdd: Wrong input data",
      221=>"NetEdit: Wrong input data",
      222=>"NetEdit: No ID specified",
      231=>"NetDelete: No ID specified",
      241=>"NetGet: No ID specified",
      242=>"NetGet: IP not found (ID: {ID})",
      311=>"IPAdd: Wrong input data",
      321=>"IPEdit: Wrong input data",
      322=>"IPEdit: No ID specified",
      331=>"IPDelete: No ID specified",
      341=>"IPGet: No ID specified",
      342=>"IPGet: IP not found (ID: {ID})",
      411=>"EventAdd: Wrong input data",
      412=>"EventAdd: \"init\" event and events starting with \"eevt_\" are predefined",
      413=>"EventAdd: Event exists (ID: {ID})",
      421=>"EventEdit: Wrong input data",
      422=>"EventEdit: \"init\" event and events starting with \"eevt_\" are predefined",
      423=>"EventEdit: No event specified",
      424=>"EventEdit: Event not found (Event: {EVENT})",
      425=>"EventEdit: Event exists (ID: {ID})",
      431=>"EventDelete: No event specified",
      432=>"EventDelete: \"init\" event and events starting with \"eevt_\" are predefined, can not be deleted",
      441=>"EventGet: No event specified",
      442=>"EventGet: Event not found (ID: {ID})",
      443=>"EventGet: Event not found (Event: {EVENT})"
    );
    
    if(!class_exists("Net_IPv4")) require_once(e_PLUGIN."netip_filter/net_ipv4.php");
    $this->netcalc = new Net_IPv4;
    
    $this->reset_event = $pref["nipf_reset_event"];
    
    require(e_PLUGIN."netip_filter/accept_events.php");
    $this->core_events = is_array($nipf_core_evts) ? $nipf_core_evts : array();
    $this->plug_events = is_array($nipf_plug_evts) ? $nipf_plug_evts : array();
  }
  
  function error($replace=false) {
    $error = $this->errors[$this->error_code];
    return is_array($replace)&&!empty($replace) ? str_replace($replace[0],$replace[1],$error) : $error;
  }
  
  
  // Get a network from NET Filter
  function get_net($id) {
    global $sql;
    
    $this->error_code = 0;
    $this->error_mess = "";
    if($id) $id = intval($id);

    if(!$id) {
      $this->error_code = 241;
      $this->error_mess = $this->error();
      return false;
    }
    
    $r = $sql->db_Select("nipf_netfilter","*","nf_id='".$id."'");
    if($r) {
      $data = $sql->db_Fetch();
      $ret = array(
        "id"         =>  $data["nf_id"],
        "bitmask"    =>  $data["nf_bitmask"],
        "netmask"    =>  $data["nf_netmask"],
        "network"    =>  $data["nf_network"],
        "broadcast"  =>  $data["nf_broadcast"],
        "netlong"    =>  $data["nf_netlong"],
        "bcastlong"  =>  $data["nf_bcastlong"],
        "netname"    =>  $data["nf_netname"],
        "addinfo"    =>  $data["nf_addinfo"],
        "active"     =>  $data["nf_active"],
        "event"      =>  $data["nf_event"],
        "denymess"   =>  $data["nf_denymess"],
      );
      return $ret;
    } else {
      $merror = mysql_error();
      if($merror) {
        $this->error_code = 124;
        $this->error_mess = $this->error(array("{MERROR}",$merror));
        return false;
      } else {
        $this->error_code = 242;
        $this->error_mess = $this->error(array("{ID}",$id));
        return false;
      }
    }
  }
  
  // Add/Edit/Delete network in NET Filter
  function manage_net($command,$id=false,$data=false) {
    global $sql, $tp;
    
    $this->error_code = 0;
    $this->error_mess = "";
    if($id) $id = intval($id);
    
    if($command=="delete") {
      if(!$id) {
        $this->error_code = 231;
        $this->error_mess = $this->error();
        return false;
      }
      
      $ret = $this->get_net($id);
      if(!$ret) return false;
      
      $r = $sql->db_Delete("nipf_netfilter","nf_id='".$id."'");
      if($r) return $r;
      else {
        $merror = mysql_error();
        if($merror) {
          $this->error_code = 123;
          $this->error_mess = $this->error(array("{MERROR}",$merror));
          return false;
        }
        else return 0;
      }
    }
    
    else if($command=="add" || $command=="edit") {
      if($command=="edit" && !$id) {
        $this->error_code = 222;
        $this->error_mess = $this->error();
        return false;
      }
      
      if($command=="edit") {
        $ret = $this->get_net($id);
        if(!$ret) return false;
      }
      
      if(!$data["net"] || !in_array($data["active"],array("y","n")) || (!is_array($data["event"]) && $data["event"]!="all" && $data["event"]!="none")) {
        $this->error_code = $command=="add" ? 211 : 221;
        $this->error_mess = $this->error();
        return false;
      }
      
      $ret = $this->netcalc->parseAddress($data["net"]);
      if(!$ret || !$ret["bitmask"] || !$ret["netmask"] || !$ret["network"] || !$ret["broadcast"]) {
        $this->error_code = $command=="add" ? 211 : 221;
        $this->error_mess = $this->error();
        return false;
      }
      
      $ret["netlong"] = $this->netcalc->ip2double($ret["network"]);
      $ret["bcastlong"] = $this->netcalc->ip2double($ret["broadcast"]);
      if(!$ret["netlong"] || !$ret["bcastlong"]) {
        $this->error_code = $command=="add" ? 211 : 221;
        $this->error_mess = $this->error();
        return false;
      }
      
      if(is_array($data["event"])) {
        if(in_array("all",$data["event"])) $data["event"] = "all";
        else if(in_array("none",$data["event"])) $data["event"] = "none";
      }
      $ins["nf_bitmask"] = $ret["bitmask"];
      $ins["nf_netmask"] = $ret["netmask"];
      $ins["nf_network"] = $ret["network"];
      $ins["nf_broadcast"] = $ret["broadcast"];
      $ins["nf_netlong"] = $ret["netlong"];
      $ins["nf_bcastlong"] = $ret["bcastlong"];
      $ins["nf_netname"] = $tp->toDB(trim($data["netname"]));
      $ins["nf_addinfo"] = $tp->toDB(trim($data["addinfo"]));
      $ins["nf_active"] = $data["active"];
      $ins["nf_event"] = is_array($data["event"]) ? implode(",",$tp->toDB($data["event"])) : $data["event"];
      $ins["nf_denymess"] = str_replace(array("\\","'","\""),array("\\\\","\'","\\\""),strip_if_magic(trim($data["denymess"])));

      if($command=="edit") {
        $r = $sql->db_Update("nipf_netfilter","nf_bitmask='".$ins["nf_bitmask"]."',nf_netmask='".$ins["nf_netmask"]."',nf_network='".$ins["nf_network"]."',nf_broadcast='".$ins["nf_broadcast"]."',nf_netlong='".$ins["nf_netlong"]."',nf_bcastlong='".$ins["nf_bcastlong"]."',nf_netname='".$ins["nf_netname"]."',nf_addinfo='".$ins["nf_addinfo"]."',nf_active='".$ins["nf_active"]."',nf_event='".$ins["nf_event"]."',nf_denymess='".$ins["nf_denymess"]."' WHERE nf_id='".$id."'");
        if($r) return $r;
        else {
          $merror = mysql_error();
          if($merror) {
            $this->error_code = 122;
            $this->error_mess = $this->error(array("{MERROR}",$merror));
            return false;
          }
          else return 0;
        }
      } else {
        $r = $sql->db_Insert("nipf_netfilter",$ins);
        if($r) return $r;
        else {
          $merror = mysql_error();
          $this->error_code = 121;
          $this->error_mess = $this->error(array("{MERROR}",$merror ? $merror : "UnKnown"));
          return false;
        }
      }
    }
    
    else {
      $this->error_code = 111;
      $this->error_mess = $this->error(array("{COMMAND}",$command));
      return false;
    }
  }
  
  // Get an IP from IP Filter
  function get_ip($id) {
    global $sql;
    
    $this->error_code = 0;
    $this->error_mess = "";
    if($id) $id = intval($id);

    if(!$id) {
      $this->error_code = 341;
      $this->error_mess = $this->error();
      return false;
    }

    $r = $sql->db_Select("nipf_ipfilter","*","ipf_id='".$id."'");
    if($r) {
      $data = $sql->db_Fetch();
      $ret = array(
        "id"        =>  $data["ipf_id"],
        "ip"        =>  $data["ipf_ip"],
        "info"      =>  $data["ipf_info"],
        "action"    =>  $data["ipf_action"],
        "event"     =>  $data["ipf_event"],
        "denymess"  =>  $data["ipf_denymess"]
      );
      return $ret;
    } else {
      $merror = mysql_error();
      if($merror) {
        $this->error_code = 124;
        $this->error_mess = $this->error(array("{MERROR}",$merror));
        return false;
      } else {
        $this->error_code = 342;
        $this->error_mess = $this->error(array("{ID}",$id));
        return false;
      }
    }
  }
  
  // Add/Edit/Delete IP in IP Filter
  function manage_ip($command,$id=false,$data=false) {
    global $sql, $tp;
    
    $this->error_code = 0;
    $this->error_mess = "";
    if($id) $id = intval($id);
    
    if($command=="delete") {
      if(!$id) {
        $this->error_code = 331;
        $this->error_mess = $this->error();
        return false;
      }
      
      $ret = $this->get_ip($id);
      if(!$ret) return false;
      
      $r = $sql->db_Delete("nipf_ipfilter","ipf_id='".$id."'");
      if($r) return $r;
      else {
        $merror = mysql_error();
        if($merror) {
          $this->error_code = 123;
          $this->error_mess = $this->error(array("{MERROR}",$merror));
          return false;
        }
        else return 0;
      }
    }
    
    else if($command=="add" || $command=="edit") {
      if($command=="edit" && !$id) {
        $this->error_code = 322;
        $this->error_mess = $this->error();
        return false;
      }
      
      if($command=="edit") {
        $ret = $this->get_ip($id);
        if(!$ret) return false;
      }

      $data["ip"] = trim($data["ip"]);
      if(!$data["ip"] || !in_array($data["action"],array("none","allow","deny")) || (!is_array($data["event"]) && $data["event"]!="all" && $data["event"]!="none")) {
        $this->error_code = $command=="add" ? 311 : 321;
        $this->error_mess = $this->error();
        return false;
      }

      $tmp = explode(".",$data["ip"]);
      $val = "";
      if(count($tmp)>4) {
        $this->error_code = $command=="add" ? 311 : 321;
        $this->error_mess = $this->error();
        return false;
      }
      foreach($tmp as $val) {
        if(!is_numeric($val) || $val<0 || $val>255) {
          $this->error_code = $command=="add" ? 311 : 321;
          $this->error_mess = $this->error();
          return false;
        }
      }

      if(is_array($data["event"])) {
        if(in_array("all",$data["event"])) $data["event"] = "all";
        else if(in_array("none",$data["event"])) $data["event"] = "none";
      }
      $ins["ipf_ip"] = $data["ip"];
      $ins["ipf_info"] = $tp->toDB(trim($data["info"]));
      $ins["ipf_action"] = $data["action"];
      $ins["ipf_event"] = is_array($data["event"]) ? implode(",",$tp->toDB($data["event"])) : $data["event"];
      $ins["ipf_denymess"] = str_replace(array("\\","'","\""),array("\\\\","\'","\\\""),strip_if_magic(trim($data["denymess"])));
  
      if($command=="edit") {
        $r = $sql->db_Update("nipf_ipfilter","ipf_ip='".$ins["ipf_ip"]."',ipf_info='".$ins["ipf_info"]."',ipf_action='".$ins["ipf_action"]."',ipf_event='".$ins["ipf_event"]."',ipf_denymess='".$ins["ipf_denymess"]."' WHERE ipf_id='".$id."'");
        if($r) return $r;
        else {
          $merror = mysql_error();
          if($merror) {
            $this->error_code = 122;
            $this->error_mess = $this->error(array("{MERROR}",$merror));
            return false;
          }
          else return 0;
        }
      } else {
        $r = $sql->db_Insert("nipf_ipfilter",$ins);
        if($r) return $r;
        else {
          $merror = mysql_error();
          $this->error_code = 121;
          $this->error_mess = $this->error(array("{MERROR}",$merror ? $merror : "UnKnown"));
          return false;
        }
      }
    }
    
    else {
      $this->error_code = 111;
      $this->error_mess = $this->error(array("{COMMAND}",$command));
      return false;
    }
  }
  
  // Get an event
  function get_event($event) {
    global $sql, $tp, $pref;
    
    $this->error_code = 0;
    $this->error_mess = "";

    if(!$event) {
      $this->error_code = 441;
      $this->error_mess = $this->error();
      return false;
    }

    $id = false;
    $pre_event = false;
    if(is_numeric($event)) $id = intval($event);
    else if($event=="init" || (!strpos($event,"eevt_") && strpos($event,"eevt_")!==false)) $pre_event = $event;
    
    if($id || !$pre_event) {
      if($id) $r = $sql->db_Select("nipf_events","*","evt_id='".$id."'");
      else $r = $sql->db_Select("nipf_events","*","evt_event='".$tp->toDB($event)."'");
      if($r) {
        $data = $sql->db_Fetch();
        $ret = array(
          "id"      =>  $data["evt_id"],
          "event"   =>  $data["evt_event"],
          "nftype"  =>  $data["evt_nftype"],
          "desc"    =>  $data["evt_desc"]
        );
        return $ret;
      } else {
        $merror = mysql_error();
        if($merror) {
          $this->error_code = 124;
          $this->error_mess = $this->error(array("{MERROR}",$merror));
          return false;
        } else {
          $this->error_code = $id ? 442 : 443;
          $this->error_mess = $this->error($id ? array("{ID}",$id) : array("{EVENT}",$event));
          return false;
        }
      }
    }
    else {
      if($pre_event!="init") $core_event = substr($pre_event,5);
      if($pre_event=="init" || in_array($core_event,$this->core_events)) {
        $ret = array(
          "id"      =>  "Predefined",
          "event"   =>  $pre_event,
          "nftype"  =>  $pref["nipf_pre_nftype"][$pre_event] ? $pref["nipf_pre_nftype"][$pre_event] : $pref["nipf_nf_type"],
          "desc"    =>  ""
        );
        return $ret;
      } else {
        $this->error_code = 443;
        $this->error_mess = $this->error(array("{EVENT}",$pre_event));
        return false;
      }
    }
  }
  
  // Add/Edit/Delete an event
  function manage_event($command,$event=false,$data=false) {
    global $sql, $tp, $pref;
    
    $this->error_code = 0;
    $this->error_mess = "";
    
    if(($command=="delete" || $command=="edit") && !$event) {
      $this->error_code = $command=="delete" ? 431 : 423;
      $this->error_mess = $this->error();
      return false;
    }

    $id = false;
    $pre_event = false;
    if($event) {
      if(is_numeric($event)) $id = intval($event);
      else if($event=="init" || (!strpos($event,"eevt_") && strpos($event,"eevt_")!==false)) $pre_event = $event;
    }

    if($command=="delete") {
      if(!$id && $pre_event) {
        $this->error_code = 432;
        $this->error_mess = $this->error();
        return false;
      }
      
      $ret = $this->get_event($id ? $id : $event);
      if(!$ret) return false;
      
      if($this->reset_event) {
        $ret["event"] = $tp->toDB($ret["event"]);
        
        $r = $sql->db_Select("nipf_netfilter","nf_id,nf_event","nf_event LIKE '%".$ret["event"].",%' OR nf_event LIKE '%,".$ret["event"]."%' OR nf_event='".$ret["event"]."'");
        if($r) {
          $nf_edit = array();
          while($nf_data = $sql->db_Fetch()) {
            $nf_id = $nf_data["nf_id"];
            $nf_event = explode(",",$nf_data["nf_event"]);
            $nf_event_key = array_search($ret["event"],$nf_event);
            unset($nf_event[$nf_event_key]);
            $nf_edit[$nf_id] = empty($nf_event) ? "none" : implode(",",$nf_event);
          }
          
          foreach($nf_event as $nf_k => $nf_v) {
            $r = $sql->db_Update("nipf_netfilter","nf_event='".$nf_v."' WHERE nf_id='".$nf_k."'");
            if(!$r) {
              $merror = mysql_error();
              if($merror) {
                $this->error_code = 122;
                $this->error_mess = $this->error(array("{MERROR}",$merror));
                return false;
              }
            }
          }
        } else {
          $merror = mysql_error();
          if($merror) {
            $this->error_code = 124;
            $this->error_mess = $this->error(array("{MERROR}",$merror));
            return false;
          }
        }
        
        $r = $sql->db_Select("nipf_ipfilter","ipf_id,ipf_event","ipf_event LIKE '%".$ret["event"].",%' OR ipf_event LIKE '%,".$ret["event"]."%' OR ipf_event='".$ret["event"]."'");
        if($r) {
          $ipf_edit = array();
          while($ipf_data = $sql->db_Fetch()) {
            $ipf_id = $ipf_data["ipf_id"];
            $ipf_event = explode(",",$ipf_data["ipf_event"]);
            $ipf_event_key = array_search($ret["event"],$ipf_event);
            unset($ipf_event[$ipf_event_key]);
            $ipf_edit[$ipf_id] = empty($ipf_event) ? "none" : implode(",",$ipf_event);
          }
          
          foreach($ipf_event as $ipf_k => $ipf_v) {
            $r = $sql->db_Update("nipf_ipfilter","ipf_event='".$ipf_v."' WHERE ipf_id='".$ipf_k."'");
            if(!$r) {
              $merror = mysql_error();
              if($merror) {
                $this->error_code = 122;
                $this->error_mess = $this->error(array("{MERROR}",$merror));
                return false;
              }
            }
          }
        } else {
          $merror = mysql_error();
          if($merror) {
            $this->error_code = 124;
            $this->error_mess = $this->error(array("{MERROR}",$merror));
            return false;
          }
        }
      }
      
      $r = $sql->db_Delete("nipf_events","evt_id='".$ret["id"]."'");
      if($r) return $r;
      else {
        $merror = mysql_error();
        if($merror) {
          $this->error_code = 123;
          $this->error_mess = $this->error(array("{MERROR}",$merror));
          return false;
        }
        else return 0;
      }
    }
    
    else if($command=="add" || $command=="edit") {
      if($command=="edit" && $pre_event) {
        if(!in_array($data["nftype"],array("all","none","all-list","none-list"))) {
          $this->error_code = 421;
          $this->error_mess = $this->error();
          return false;
        }
        
        if($pre_event!="init") {
          $core_event = substr($pre_event,5);
          if(!in_array($core_event,$this->core_events)) {
            $this->error_code = 424;
            $this->error_mess = $this->error(array("{EVENT}",$pre_event));
            return false;
          }
        }
      } else {
        $data["event"] = trim($data["event"]);
        if(!$data["event"] || $data["event"]=="none" || $data["event"]=="all" || !in_array($data["nftype"],array("all","none","all-list","none-list"))) {
          $this->error_code = $command=="add" ? 411 : 421;
          $this->error_mess = $this->error();
          return false;
        }
      
        if($data["event"]=="init" || (!strpos($data["event"],"eevt_") && strpos($data["event"],"eevt_")!==false)) {
          $this->error_code = $command=="add" ? 412 : 422;
          $this->error_mess = $this->error();
          return false;
        }
      
        $ins["evt_event"] = $tp->toDB($data["event"]);
        $ins["evt_nftype"] = $data["nftype"];
        $ins["evt_desc"] = $tp->toDB(trim($data["desc"]));
      }
      
      if($command=="edit") {      
        if($id || !$pre_event) {
          $ret = $this->get_event($id ? $id : $event);
          if(!$ret) return false;
        
          $r = $sql->db_Select("nipf_events","evt_id","evt_event='".$ins["evt_event"]."' AND evt_id!='".$ret["id"]."'");
          if($r) {
            $data = $sql->db_Fetch();
            $this->error_code = 425;
            $this->error_mess = $this->error(array("{ID}",$data["evt_id"]));
            return false;
          }
          else {
            $merror = mysql_error();
            if($merror) {
              $this->error_code = 124;
              $this->error_mess = $this->error(array("{MERROR}",$merror));
              return false;
            }
          }
          
          $r = $sql->db_Update("nipf_events","evt_event='".$ins["evt_event"]."',evt_nftype='".$ins["evt_nftype"]."',evt_desc='".$ins["evt_desc"]."' WHERE evt_id='".$ret["id"]."'");
          if($r) return $r;
          else {
            $merror = mysql_error();
            if($merror) {
              $this->error_code = 122;
              $this->error_mess = $this->error(array("{MERROR}",$merror));
              return false;
            }
            else return 0;
          }
        } else {
          $pref["nipf_pre_nftype"][$pre_event] = $data["nftype"];
          save_prefs();
          return true;
        }
      } else {
        $r = $sql->db_Select("nipf_events","evt_id","evt_event='".$ins["evt_event"]."'");
        if($r) {
          $data = $sql->db_Fetch();
          $this->error_code = 413;
          $this->error_mess = $this->error(array("{ID}",$data["evt_id"]));
          return false;
        }
        else {
          $merror = mysql_error();
          if($merror) {
            $this->error_code = 124;
            $this->error_mess = $this->error(array("{MERROR}",$merror));
            return false;
          }
        }
        
        $r = $sql->db_Insert("nipf_events",$ins);
        if($r) return $r;
        else {
          $merror = mysql_error();
          $this->error_code = 121;
          $this->error_mess = $this->error(array("{MERROR}",$merror ? $merror : "UnKnown"));
          return false;
        }
      }
    }
    
    else {
      $this->error_code = 111;
      $this->error_mess = $this->error(array("{COMMAND}",$command));
      return false;
    }
  }
}
?>